/******************************************************************************
 * @file      drv_qfc41d.c
 * @brief     drv_qfc41d driver
 * @version   V1.0
 * @author    Gary
 * @copyright
 ******************************************************************************/

#include "drv_qfc41d.h"

uint8_t rev_buff[QFC41D_MAX_RECV_SIZE] = {0};
uint8_t send_buff[QFC41D_MAX_SEND_SIZE] = {0};
char wifiName[40] = "BLUESUNESS", wifiPassWord[40] = "bluesun009", bleName[40] = "bsm1";

char cmd[1000] = {0}; //ջռʹ

#define TX_DMA_BUFFER_SIZE 1000

uint8_t dma_tx_buffer[TX_DMA_BUFFER_SIZE];
volatile uint8_t dma_tx_busy = 0;

//
WifiBleItem qfc41dItem = {
    .dev = QFC41D_UART_PORT,
    .buf_pos = 0,
    .buf_size = QFC41D_MAX_RECV_SIZE,
    .buf = rev_buff,
    .sendCall  = NULL,
};

static void drv_qfc41d_push_data(WifiBleItem* item, uint8_t *buf, uint16_t len)
{
    uint16_t i = 0;

    if((item != NULL) && (item->buf_pos + len < item->buf_size))
    {
        item->buf[item->buf_pos++] = buf[i];
        // зʾһ AT ָ
        if (buf[i] == '\n')
        {
            item->buf[item->buf_pos] = '\0';   // ӽ
            item->buf_pos = 0;  			   // ׼һ
        }
    }
}


static void drv_qf41d_uart_rx_it_call(kit_ret_e res, void *data)
{
    OS_TRACE_ISR_ENTER();
    drv_qfc41d_push_data(&qfc41dItem, data, 1);
    OS_TRACE_ISR_EXIT();
}

// ͵ַ
void drv_wireless_send_char(char c)
{
    while (!(USART2->SR & USART_SR_TXE));  // ȴͻ
    USART2->DR = c;
}

// ַ
void drv_wireless_send_string(const char *str)
{
    while (*str)
    {
        drv_wireless_send_char(*str++);
    }
    drv_wireless_send_char('\r');  //  CR
    drv_wireless_send_char('\n');  //  LF
}

void drv_wireless_send_string_dma(const char *str)
{
		bsp_task_delay_ms(250);
    if (dma_tx_busy) return;  // ڷͣŶ

    uint16_t len = 0;

    // ݵ DMA 
    while (*str && len < TX_DMA_BUFFER_SIZE - 2) {
        dma_tx_buffer[len++] = *str++;
    }

    //  \r\n
    dma_tx_buffer[len++] = '\r';
    dma_tx_buffer[len++] = '\n';

    //  DMA 
    DMA1_Stream6->CR &= ~DMA_SxCR_EN;
    while (DMA1_Stream6->CR & DMA_SxCR_EN);

    DMA1_Stream6->M0AR = (uint32_t)dma_tx_buffer;
    DMA1_Stream6->NDTR = len;

    dma_tx_busy = 1;
    DMA1_Stream6->CR |= DMA_SxCR_EN;
    USART2->CR3 |= USART_CR3_DMAT;  //  DMA 
}


//  AT ָȴֵ
int drv_at_send_cmd(const char *cmd, const char *response, uint32_t timeout)
{
    memset(qfc41dItem.buf, 0, QFC41D_MAX_RECV_SIZE);
    qfc41dItem.buf_pos = 0;  // սջ

    drv_wireless_send_string(cmd);

    uint32_t start_time = kit_time_get_tick();  // ȡǰʱ
    while (kit_time_get_tick() - start_time < timeout)
    {
        if (strstr((char *)qfc41dItem.buf, response))
        {
            return 1;  // ɹƥ䷵ֵ
        }
    }
    return 0;  // ʱδյ
}

static uint8_t drv_wifi_init()
{
    // 1. ģ鲨
	/*
    if (!drv_at_send_cmd(QFC41D_SET_BATE, "OK", WIFI_TIMEOUT))
    {
        KIT_PRINTF("WiFi Module not responding!\r\n");
        return 1 ;
    }
	*/
	
    // 2.  AT ָ
    if (!drv_at_send_cmd("AT", "OK", WIFI_TIMEOUT))
    {
//        KIT_PRINTF("WiFi Module not responding!\r\n");
        return 1 ;
    }
		
		if (!drv_at_send_cmd("AT+QSTASTOP", "OK", WIFI_TIMEOUT))
    {
//        KIT_PRINTF("Failed to set WiFi mode!\r\n");
        return 1;
    }
		
		if (!drv_at_send_cmd("AT+QSOFTAPSTOP", "OK", WIFI_TIMEOUT))
    {
//        KIT_PRINTF("Failed to set WiFi mode!\r\n");
        return 1;
    }
		
//		if (!drv_at_send_cmd("AT+QSTAAPINFO=BLUESUNESS,bluesun009", "OK", WIFI_CONNECT_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to set WiFi mode!\r\n");
//        return 1;
//    }
//		kit_time_dly_ms(2000);
		
//		if (!drv_at_send_cmd("AT+QSTAAPINFO=BLUESUNESS,bluesun009", "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to set WiFi mode!\r\n");
//        return 1;
//    }
	
    // 3.  Wi-Fi Ϊ DHCPʹ
//    if (!drv_at_send_cmd(QFC41D_SET_DHCP_ENABLE, "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to set WiFi mode!\r\n");
//        return 1;
//    }

    // 4.  Wi-Fi 	
//    snprintf(cmd, sizeof(cmd), "AT+QSTAAPINFODEF=%s,%s", wifiName, wifiPassWord);
//    if (!drv_at_send_cmd(cmd, "WIFI CONNECTED", WIFI_TIMEOUT * 2))
//    {
//        KIT_PRINTF("Failed to connect to WiFi!\r\n");
//        //return 1; //󲻷-ֹӰִ
//    }

//    // 5. ȡ IP ַ
//    if (!drv_at_send_cmd("AT+QGETIP=station", "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to get IP address!\r\n");
//        //return 1;  //󲻷-ֹӰִ
//    }

//    KIT_PRINTF("WiFi Initialization Successful!\r\n");
	return 0;
}

uint8_t drv_ble_init()
{
	
    // 1. ʼ,Ϊ
//    if (!drv_at_send_cmd("AT+QBLEINIT=3", "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to power off BLE!\r\n");
//        return 1;
//    }
//	
//    // 2. 
//    snprintf(cmd, sizeof(cmd), "AT+QBLENAME=%s", bleName);
//    if (!drv_at_send_cmd(cmd, "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to set BLE name!\r\n");
//        return 1;
//    }

//    // 3. ù㲥
//    if (!drv_at_send_cmd("AT+QBLEADVPARAM=2048,2048", "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to enable BLE!\r\n");
//        return 1;
//    }
//	
//    // 4. ù㲥
//    if (!drv_at_send_cmd("AT+QBLEADVDATA=06094643343144", "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to enable BLE!\r\n");
//        return 1;
//    }

//    // 5. 
//    if (!drv_at_send_cmd("AT+QBLEGATTSSRV=FFF0", "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to set BLE role!\r\n");
//        return 1;
//    }
//	
//    // 6. ֵ
//    if (!drv_at_send_cmd("AT+QBLEGATTSSRV=FFF1", "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to set BLE role!\r\n");
//        return 1;
//    }

//    // 7. ʼ㲥
//    if (!drv_at_send_cmd("AT+QBLEADVSTART", "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to start BLE advertising!\r\n");
//        return 1;
//    }

//    KIT_PRINTF("BLE Initialization Successful!\r\n");

		
		if (!drv_at_send_cmd("AT+QBLEINIT=3", "OK", WIFI_TIMEOUT))
    {
//        KIT_PRINTF("Failed to set BLE role!\r\n");
        return 1;
    }

    //wifiƣBSMͷ׺Լsn
    if (!drv_at_send_cmd("AT+QBLENAME=BSM11223344", "OK", WIFI_TIMEOUT))
    {
//        KIT_PRINTF("Failed to start BLE advertising!\r\n");
        return 1;
    }
		
    return 0;
}


uint8_t drv_qfc41d_init(void)
{
    //ʼ
    drv_uart_init((UartDev)qfc41dItem.dev, 115200, 0 | UART_CFG_STOP_BIT_1, kGpioType_WIRELESSUart_Tx, kGpioType_WIRELESSUart_Rx);
	
	USART2->CR1 = USART_CR1_TE | USART_CR1_RE | USART_CR1_RXNEIE | USART_CR1_UE;
    //ýջص
    drv_uart_set_interrupt(QFC41D_UART_PORT, kUartInterrupt_Rx, APP_CFG_INT_PRIO_UART2_RX, drv_qf41d_uart_rx_it_call);//#define SIM_UART_PORT     	kUartDev_2

		//ʼwifi
//		if(drv_wifi_init() != 0)
//		{
//			return 1;   
//		}

		//ʼBLE
    if (drv_ble_init() != 0)
    {
        return 1;
    }


	return 0;
}

#define MQTT_BROKER     "47.120.14.45"   //  MQTT ַ
#define MQTT_PORT       3011               // MQTT ˿
#define MQTT_CLIENT_ID  "fc41d_client"       // ͻ ID
#define MQTT_USER       "user"               // MQTT û
#define MQTT_PASSWORD   "password"           // MQTT 
#define MQTT_TOPIC      "bluesun/bms/period/11223344"        // /
#define MQTT_MESSAGE    "Hello from FC41D!"  // ͵Ϣ

//  MQTT 
//uint8_t drv_mqtt_connect()
//{
//	
//    //1 MQTT Эİ汾Ϊ V4
////    snprintf(cmd, sizeof(cmd), "AT+QMTCFG=\"version\",1,4");
////    if (!drv_at_send_cmd(cmd, "OK", WIFI_TIMEOUT))
////    {
////        KIT_PRINTF("Failed to set MQTT server!\r\n");
////        return 1;
////    }
//	
//	//2 MQTT ͻ֮ĻỰ
//    snprintf(cmd, sizeof(cmd), "AT+QMTOPEN=0,\"%s\",%d",MQTT_BROKER,MQTT_PORT);
//    if (!drv_at_send_cmd(cmd, "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to set MQTT server!\r\n");
//        return 1;
//    }
//	

//    //2MQTT ͻ֮佨 MQTT  AT+QMTCONN=<clientID>,<client_identity>,<username>,<password>
//    //snprintf(cmd, sizeof(cmd), "AT+QMTCONN=0,%s,%s,%s",MQTT_CLIENT_ID,MQTT_USER, MQTT_PASSWORD);
//    if (!drv_at_send_cmd("AT+QMTCONN=0,\"clientBCUwifi03\"", "OK", WIFI_TIMEOUT))
//    {
//        KIT_PRINTF("Failed to set MQTT client credentials!\r\n");
//        return 1;
//    }

//    KIT_PRINTF("MQTT Connection Successful!\r\n");
//    return 0;
//}

//  MQTT 
static uint8_t drv_mqtt_subscribe()
{
    snprintf(cmd, sizeof(cmd), "AT+MQTTSUB=0,1,\"%s\",2", MQTT_TOPIC);
    drv_wireless_send_string(cmd);
    return 0;
}

//  MQTT Ϣ 
/*
 * AT+QMTPUB=<clientID>,<msgID>,<QoS>,<retain>,<topic>,<payload_len gth>,<payload>
 * length Χ0~1500
 * str mqttjson
 */
uint8_t drv_mqtt_publish(char* str,uint16_t length)
{
    snprintf(cmd, sizeof(cmd), "AT+QMTPUB=0,1,2,0,%s,%d,%s",MQTT_TOPIC, length, str);
    if (!drv_at_send_cmd(cmd, "OK\0", WIFI_TIMEOUT))
    {
//        KIT_PRINTF("Failed to publish message!\r\n");
        return 1;
    }

//    KIT_PRINTF("MQTT Message Published Successfully!\r\n");
    return 0;
}

uint8_t drv_mqtt_publish_no_respose(char* str,uint16_t length)
{
//    snprintf(cmd, sizeof(cmd), "AT+QMTPUB=0,1,2,0,%s,%d,%s",MQTT_TOPIC, length, str);
		snprintf(cmd, sizeof(cmd), "AT+QMTPUB=0,1,2,0,\"%s\",%d,\"%s\"", MQTT_TOPIC, length, str);
//    drv_wireless_send_string(cmd);  // δʹDMA
		drv_wireless_send_string_dma(cmd);  // ʹDMA
//		kit_time_dly_ms(100);
    return 0;
}

// MQTT ʼ
//uint8_t drv_mqtt_init()
//{
//    if (drv_mqtt_connect() != 0)
//    {
//        return 1;
//    }

//    if (drv_mqtt_subscribe() != 0)
//    {
//        return 1;
//    }

//    return 0;
//}
